package appia.protocols.stf.rrpc;

import appia.*;
import appia.events.*;
import appia.events.channel.*;
import appia.message.*;
import appia.protocols.udpsimple.RegisterSocketEvent;
import appia.protocols.udpsimple.InetWithPort;
import appia.protocols.remoteView.*;
import appia.protocols.group.*;

import java.io.*;
import java.util.*;
import java.net.InetAddress;


/**
 * Classe que tem o codigo responsavel
 * pela parte generica do seistema RRPC
 * no lado do cliente.
 * Nesta classe vao ter de fazer,
 * por exemplo, o codigo responsavel 
 * pela filtragem de respostas repetidas.
 */
public class RRPCSession extends Session {
    
    //referencia para o canal
    public  Channel       channel=null;

    //array para guardar as moradas dos servidores
    public InetWithPort serverAddrs[];
    

    public RRPCSession(RRPCLayer l) {
	super(l);	
    }
    
    
    /**
     * metodo chamado quando esta camadas recebe um evento
     */    
    public void handle(Event e) {
	if(e instanceof ChannelInit)
	    handleChannelInit((ChannelInit)e);	
	if(e instanceof RemoteViewEvent){
	    handleRemoteViewEvent((RemoteViewEvent)e);
	    return;
	}
	if(e instanceof SendableEvent)
	    handleSendableEvent((SendableEvent)e);
	
    }

    /*
     * trata o ChannelInit. 
     * Mensagem de inicializacao do canal.   
     * Primeira mensagem recebida e so depois
     * se pode enviar mensagens.
     */
    private void handleChannelInit(ChannelInit e) {

	try {
	    e.go();
	}
	catch(AppiaEventException ex) {
	    System.err.println("Unexpected exception in RRPC "+
			       "session");
	}

	channel=e.getChannel();

	try{
	    RemoteViewEvent rve=new RemoteViewEvent(channel,new Direction(Direction.DOWN),this,new Group("groupID"));	    

	    rve.init();
	    rve.go();
	}
	catch(AppiaEventException ex){
	    ex.printStackTrace();
	    System.out.println("ERROR sending event");
	}
	
    }

    /*
     * Trata o Sendable event recebido
     * Por enquanto apenas o envia para cima.
     */
    private void handleSendableEvent(SendableEvent e){
	try{
	    e.go();
	}
	catch(AppiaEventException ex){
	    System.err.println("ERROR sending event");
	}
    }

    /**
     * Trata um evento do tipo de pedido de vista.
     */
    public void handleRemoteViewEvent(RemoteViewEvent e){
	System.out.println("recebeu e guardou uma Remote View");

	serverAddrs=e.getAddresses();	
	/*	for(int i=0;i!=serverAddrs.length;i++)
		System.out.println("host["+i+"]= "+serverAddrs[i].host+":"+serverAddrs[i].port);*/

    }

    /**
     * Retorna a Layer desta Session
     */
    public Layer getLayer() {
	return layer;
    }
}
